;;; sysmon.el --- System Monitor on Emacs
;;;
;;; Version: 2016/12/11
;;;
;;; Install
;;;  copy sysmon.el to /usr/share/emacs/site-lisp/.
;;;
;;; Setting
;;;  add to .emacs
;;;  ----
;;;  (require 'sysmon)
;;;  ----
;;;
;;; Start and Stop
;;;  M-x sysmon-start
;;;  M-x sysmon-stop
;;;

;;; Code:

(defcustom sysmon-plotchar        ?|
  "plot charactor" :group 'sysmon)
(defcustom sysmon-update-interval 1.0
  "update interval" :group 'sysmon)

(defvar sysmon-buffername "*System-Monitor*"
  "sysmon buffer name")
(defvar sysmon-timer nil
  "sysmon timer")

(defvar sysmon-mode-line-format
  '("    "
    mode-line-frame-identification
    mode-line-buffer-identification))

(defvar sysmon-prev_cpustat nil)
(defvar sysmon-rate-buffer  nil)
(defvar sysmon-disable-fitwindow nil)

(defun sysmon-start ()
  "start sysmon"
  (interactive)
  (progn
    (if (file-readable-p "/proc/stat")
        (progn
          (setq sysmon-rate-buffer (list 0.0))
          (setq sysmon-prev_cpustat nil)
          (if sysmon-timer (cancel-timer sysmon-timer))
          (setq sysmon-timer (run-with-timer t sysmon-update-interval 'sysmon-main))
          (let ((selected))
            (setq selected (selected-window))
            (switch-to-buffer-other-window sysmon-buffername)
            (select-window selected)))
      (message "Can not open /proc/stat")
      )
    ))

(defun sysmon-stop ()
  "stop sysmon"
  (interactive)
  (if sysmon-timer
      (progn
        (cancel-timer sysmon-timer)
        (setq sysmon-timer nil)
        (kill-buffer sysmon-buffername))))

(defun sysmon-main ()
  (with-current-buffer (get-buffer-create sysmon-buffername)
    (let ((linebuf))

      (setq truncate-lines t)
      (setq cursor-type nil)
      (read-only-mode 0)
      (buffer-disable-undo)
      (erase-buffer)

      (with-temp-buffer
        (insert-file-contents "/proc/stat")
        (setq linebuf (split-string (buffer-string) "\n")))

      (dolist (line linebuf)
        (if (string-match "^cpu " line)
            (let* ((field (split-string line))
                   (cpustat (list (float (string-to-number (nth 1 field)))
                                  (float (string-to-number (nth 2 field)))
                                  (float (string-to-number (nth 3 field)))
                                  (float (string-to-number (nth 4 field))))))

              (if sysmon-prev_cpustat
                  (let* ((dif_user (- (nth 0 cpustat) (nth 0 sysmon-prev_cpustat)))
                         (dif_nice (- (nth 1 cpustat) (nth 1 sysmon-prev_cpustat)))
                         (dif_sys  (- (nth 2 cpustat) (nth 2 sysmon-prev_cpustat)))
                         (dif_idle (- (nth 3 cpustat) (nth 3 sysmon-prev_cpustat)))
                         (sum (+ dif_user dif_nice dif_sys dif_idle))
                         )
                    (insert
                     (format "total: %6.2f%% sys: %6.2f%%\n"
                             (/ (* 100.0 (+ dif_user dif_nice dif_sys)) sum)
                             (/ (* 100.0 dif_sys ) sum)))
                    (insert (format "%s" (sysmon-build-graph (/ (* 100.0 (+ dif_user dif_nice dif_sys)) sum))))
                    ))
              (setq sysmon-prev_cpustat cpustat)
              )
          ))
      (goto-char (point-min))
      (setq mode-line-format sysmon-mode-line-format)
      (read-only-mode 1)
      )
    ))

(defun sysmon-build-graph (load-rate)
  (let ((canvas-w
         (if (or (eq (window-system) nil) sysmon-disable-fitwindow)
             (window-body-width  (get-buffer-window sysmon-buffername))
           (/ (* (frame-char-width)
                 (window-body-width  (get-buffer-window sysmon-buffername)))
              (default-font-width))))
        (canvas-h
         (if (or (eq (window-system) nil) sysmon-disable-fitwindow)
             (- (window-body-height (get-buffer-window sysmon-buffername)) 1)
           (- (/ (* (frame-char-height)
                    (window-body-height (get-buffer-window sysmon-buffername)))
                 (default-font-height))
              1)))
        (canvas ""))

    (setq sysmon-rate-buffer (last (append sysmon-rate-buffer (list load-rate)) (- canvas-w 1)))
    (if (and (< 0 canvas-w) (< 0 canvas-h))
        (let ((line)
              (posx 0))
          (dotimes (n (- canvas-w 1))
            (setq line (concat line " ")))
          (setq line (concat line "\n"))
          (dotimes (n canvas-h)
            (setq canvas (concat canvas line)))

          (dolist (rate sysmon-rate-buffer)
            (dotimes (n (* (float canvas-h) (/ rate 100.0)))
              (aset canvas (+ (* (- (- canvas-h 1) n) canvas-w) posx) sysmon-plotchar)
              )
            (setq posx (+ posx 1))
            )
          ))
    canvas
    ))

(provide 'sysmon)
